@@ -1,5 +1,5 @@
 package MooX::Role::POE::Emitter;
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 use Moo::Role;
 
@@ -12,6 +12,7 @@ use MooX::Types::MooseLike::Base qw/:all/;
 
 use MooX::Role::Pluggable::Constants;
 
+sub E_TAG () { "Emitter Running" }
 
 ##
 use namespace::clean -except => 'meta';
@@ -21,36 +22,52 @@ with 'MooX::Role::Pluggable';
 
 
 has 'alias' => (
-  lazy => 1,
-  is   => 'ro',
-  isa  => Str,
+  lazy      => 1,
+  is        => 'ro',
+  isa       => Str,
   predicate => 'has_alias',
   writer    => 'set_alias',
   default   => sub { "$_[0]" },
 );
 
 has 'event_prefix' => (
-  lazy => 1,
-  is   => 'ro',
-  isa  => Str,
+  lazy      => 1,
+  is        => 'ro',
+  isa       => Str,
   predicate => 'has_event_prefix',
   writer    => 'set_event_prefix',
   default   => sub { "emitted_" },
 );
 
+has 'pluggable_type_prefixes' => (
+  ## Optionally remap PROCESS / NOTIFY types
+  lazy      => 1,
+  is        => 'ro',
+  isa       => HashRef,
+  predicate => 'has_pluggable_type_prefixes',
+  writer    => 'set_pluggable_type_prefixes',
+  default   => sub {
+   +{ 
+      PROCESS => 'P',
+      NOTIFY  => 'N',
+    }
+  },
+);
+
 has 'object_states' => (
-  lazy => 1,
-  is   => 'ro',
-  isa  => ArrayRef,
+  lazy      => 1,
+  is        => 'ro',
+  isa       => ArrayRef,
   predicate => 'has_object_states',
   writer    => 'set_object_states',
   trigger   => 1,
+  default   => sub { [] },
 );
 
 has 'register_prefix' => (
-  lazy => 1,
-  is   => 'ro',
-  isa  => Str,
+  lazy      => 1,
+  is        => 'ro',
+  isa       => Str,
   predicate => 'has_register_prefix',
   writer    => 'set_register_prefix',
   ## Emitter_register / Emitter_unregister
@@ -58,39 +75,30 @@ has 'register_prefix' => (
 );
 
 has 'session_id' => (
-  init_arg => undef,
-  lazy => 1,
-  is   => 'ro',
-  isa  => Defined,
+  init_arg  => undef,
+  lazy      => 1,
+  is        => 'ro',
+  isa       => Defined,
   predicate => 'has_session_id',
   writer    => 'set_session_id',
-);
-
-has 'pluggable_type_prefixes' => (
-  ## Optionally remap PROCESS / NOTIFY types
-  lazy => 1,
-  is   => 'ro',
-  isa  => HashRef,
-  predicate => 'has_pluggable_type_prefixes',
-  writer    => 'set_pluggable_type_prefixes',
-  default   => sub { { PROCESS => 'P', NOTIFY => 'N' } },
+  default   => sub { -1 },
 );
 
 
 has '__emitter_reg_sessions' => (
   ## ->{ $session_id } = { refc => $ref_count, id => $id };
-  lazy => 1,
-  is   => 'ro',
-  isa  => HashRef,
-  default => sub { {} },
+  lazy    => 1,
+  is      => 'ro',
+  isa     => HashRef,
+  default => sub { +{} },
 );
 
 has '__emitter_reg_events' => (
   ## ->{ $event }->{ $session_id } = 1
-  lazy => 1,
-  is   => 'ro',
-  isa  => HashRef,
-  default => sub { {} },
+  lazy    => 1,
+  is      => 'ro',
+  isa     => HashRef,
+  default => sub { +{} },
 );
 
 
@@ -126,24 +134,25 @@ sub _start_emitter {
         '_stop'            => '__emitter_stop',
         'shutdown_emitter' => '__shutdown_emitter',
 
-        'register'    => '__emitter_register',
-        'subscribe'   => '__emitter_register',
-        'unregister'  => '__emitter_unregister',
-        'unsubscribe' => '__emitter_unregister',
+        'register'         => '__emitter_register',
+        'subscribe'        => '__emitter_register',
+        'unregister'       => '__emitter_unregister',
+        'unsubscribe'      => '__emitter_unregister',
+        'emit'             => '__emitter_notify',
 
-        'emit'        => '__emitter_notify',
-
-        '_default'    => '__emitter_disp_default',
-        '__emitter_really_default' => '_emitter_default',
+        '_default'               => '__emitter_disp_default',
+        '__emitter_real_default' => '_emitter_default',
       },
 
       $self => [ qw/
+
         __emitter_notify
 
         __emitter_timer_set
         __emitter_timer_del
 
-        _emitter_sigdie
+        __emitter_sigdie
+
       / ],
 
       (
@@ -289,7 +298,7 @@ sub _trigger_object_states {
 
     for my $ev (@$evarr) {
       confess "Disallowed handler: $ev"
-        if grep { $_ eq $ev } @disallowed;
+        if grep {; $_ eq $ev } @disallowed;
     }
 
   }
@@ -331,7 +340,7 @@ sub __emitter_drop_sessions {
     my $count = $self->__get_ses_refc($id);
 
     $poe_kernel->refcount_decrement(
-      $id, 'Emitter running'
+      $id, E_TAG
     ) while $count-- > 0;
 
     delete $self->__emitter_reg_sessions->{$id}
@@ -392,7 +401,7 @@ sub __emitter_start {
 
   $self->set_session_id( $session->ID );
 
-  $kernel->sig('DIE', '_emitter_sigdie' );
+  $kernel->sig('DIE', '__emitter_sigdie' );
 
   $kernel->alias_set( $self->alias );
 
@@ -402,12 +411,12 @@ sub __emitter_start {
     ## Have a parent session.
 
     ## refcount for this session.
-    $kernel->refcount_increment( $s_id, 'Emitter running' );
+    $kernel->refcount_increment( $s_id, E_TAG );
     $self->__incr_ses_refc( $s_id );
     $self->__reg_ses_id( $s_id );
 
     ## subscribe parent session to all notification events.
-    $self->__emitter_reg_events->{ 'all' }->{ $s_id } = 1;
+    $self->__emitter_reg_events->{all}->{ $s_id } = 1;
 
     ## Detach child session.
     $kernel->detach_myself;
@@ -425,11 +434,16 @@ sub __emitter_disp_default {
   if (ref $event eq 'CODE') {
     ## Anonymous coderef callback.
     ## Cute trick from dngor:
+    ##  - Shove arguments back into @_
+    ##    (starting at ARG0 and replacing ARG0/ARG1)
+    ##  - Set $_[STATE] to our coderef
+    ##    (callback sub can retrieve itself via $_[STATE])
+    ##  - Replace current subroutine
     splice @_, ARG0, 2, @$args;
     $_[STATE] = $event;
     goto $event
   } else {
-    $self->call( '__emitter_really_default', $event, $args );
+    $self->call( '__emitter_real_default', $event, $args );
   }
 }
 
@@ -445,7 +459,7 @@ sub _emitter_default {
     or $event =~ /^emitter_(?:started|stopped)$/ ;
 }
 
-sub _emitter_sigdie {
+sub __emitter_sigdie {
   my ($kernel, $self) = @_[KERNEL, OBJECT];
   my $exh = $_[ARG1];
 
@@ -510,7 +524,7 @@ sub __emitter_register {
 
     ## Make sure registered session hangs around
     ##  (until _unregister or shutdown)
-    $kernel->refcount_increment( $s_id, 'Emitter running' )
+    $kernel->refcount_increment( $s_id, E_TAG )
       unless $s_id eq $self->session_id
       or $self->__get_ses_refc($s_id);
 
@@ -531,7 +545,7 @@ sub __emitter_unregister {
   EV: for my $event (@events) {
     unless (delete $self->__emitter_reg_events->{$event}->{$s_id}) {
       ## Possible we should just not give a damn?
-      warn "Cannot unregister $event for $s_id -- not registered";
+      warn "Cannot unregister $event for $s_id -- not registered\n";
       next EV
     }
 
@@ -545,7 +559,7 @@ sub __emitter_unregister {
       ## No events left for this session.
       delete $self->__emitter_reg_sessions->{$s_id};
 
-      $kernel->refcount_decrement( $s_id, 'Emitter running' )
+      $kernel->refcount_decrement( $s_id, E_TAG )
         unless $_[SESSION] == $sender;
     }
 
@@ -576,7 +590,7 @@ MooX::Role::POE::Emitter - Pluggable POE event emitter role for cows
     $self->set_object_states(
       [
         $self => {
-          ## ... Add some extra handlers ...
+          ## Add some extra handlers to your Emitter
           'emitter_started' => '_emitter_started',
         },
 
@@ -645,24 +659,41 @@ MooX::Role::POE::Emitter - Pluggable POE event emitter role for cows
 
 This is a L<Moo::Role> for a L<POE> Observer Pattern implementation; 
 it is derived from L<POE::Component::Syndicator> by BINGOS, HINRIK, APOCAL 
-et al, but with more cows ;-)
+et al, but with more cows ;-) plus a few extra features and a slightly 
+faster dispatcher.
 
 Consuming this role gives your class a L<POE::Session> capable of 
-emitting events to loaded plugins and registered "listener" sessions. It 
-also brings in L<MooX::Role::Pluggable>, making your emitter pluggable (see the 
+emitting events to loaded plugins and registered "listener" sessions.
+
+The Emitter role consumes L<MooX::Role::Pluggable>, 
+making your emitter pluggable (see the 
 L<MooX::Role::Pluggable> documentation for plugin-related details).
 
 You do not need to create your own L<POE::Session>; calling 
 L</_start_emitter> will spawn one for you.
+You also get some useful sugar over POE event dispatch (such as anonymous 
+coderef callbacks); see L</Methods>.
 
 =head2 Creating an Emitter
 
 L</SYNOPSIS> contains an emitter that uses B<set_$attrib> methods to
 configure itself when C<spawn()> is called; these attribs can, of course,
-be set when your Emitter is constructed.
+be set when your Emitter is constructed:
+
+  my $emitter = MyEmitter->new(
+    alias => 'my_emitter',
+    pluggable_type_prefixes => {
+      NOTIFY  => 'Notify',
+      PROCESS => 'Proc',
+    },
+    # . . .
+  );
 
 =head3 Attributes
 
+Most of these can be altered via B<set_$attrib> methods at any time before 
+L</_start_emitter> is called.
+
 =head4 alias
 
 B<alias> specifies the POE::Kernel alias used for our L<POE::Session>; 
@@ -880,13 +911,29 @@ managing timers within the context of the emitter's L<POE::Session>.
 Provides an interface to L<POE::Kernel>'s yield/post() method, dispatching 
 POE events within the context of the emitter's session.
 
-The event can be either a named event/state or an anonymous coderef:
+The event can be either a named event/state dispatched to your Emitter's 
+L<POE::Session>:
 
-  $self->yield( sub {
-    my ($kernel, @args) = @_[KERNEL, ARG0 .. $#_];
-    . . .
+  $emitter->yield( 'some_event', @args );
+
+... or an anonymous coderef, which is executed as if it were a named 
+POE state belonging to your Emitter:
+
+  $emitter->yield( sub {
+    ## $_[OBJECT] is the Emitter's object:
+    my ($kernel, $self) = @_[KERNEL, OBJECT];
+    my @params          = @_[ARG0 .. $#_];
+
+    ## $_[STATE] is the current coderef
+    ## Yield ourselves again, for example:
+    $self->yield( $_[STATE], @new_args )
+      if $some_condition;
   }, $some, $args );
 
+Inside an anonymous coderef callback such as shown above, C<$_[OBJECT]> is 
+the Emitter's C<$self> object and C<$_[STATE]> contains the callback 
+coderef itself.
+
 =head3 call
 
   $self->call( $poe_event, @args );
@@ -921,13 +968,23 @@ A prefixed (L</event_prefix>) 'timer_deleted' event is emitted when a timer
 is deleted. Arguments are the removed alarm ID, the event name or coderef, 
 and any event parameters, respectively.
 
+=head2 Moose compatibility
+
+This Role is Moose-compatible as of version 0.07, but you'll need to 
+consume L<MooX::Role::Pluggable> on its own, as far as I can tell:
+
+  package MyEmitter;
+  use Moose;
+  with 'MooX::Role::Pluggable';
+  with 'MooX::Role::POE::Emitter';
+
 =head1 AUTHOR
 
 Jon Portnoy <avenj@cobaltirc.org>
 
 Derived from L<POE::Component::Syndicator>-0.06 by BINGOS, HINRIK, 
 APOCAL et al. That will probably do you for non-Moo(se) use cases; I 
-needed something cow-like that worked with L<MooX::Role::Pluggable>.
+needed something cow-like that worked with L<MooX::Role::Pluggable>. 
 
 Licensed under the same terms as perl5
 
